unit Test;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, Spin, ComCtrls, WavUtils, ExtCtrls, TeeProcs, TeEngine,
  Chart, Series;

type
  TTestForm = class(TForm)
    StatusBar: TStatusBar;
    ControlBox: TScrollBox;
    WaveletOrder: TSpinEdit;
    WaveletOrderLabel: TLabel;
    DecompLevelLabel: TLabel;
    DecompLevel: TSpinEdit;
    Analyze: TButton;
    Synthesize: TButton;
    ChartBox: TScrollBox;
    MainMenu: TMainMenu;
    mnuFile: TMenuItem;
    mniLoad: TMenuItem;
    mniSave: TMenuItem;
    mniExit: TMenuItem;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    procedure FormDestroy(Sender: TObject);
    procedure FileOpenAccept(Sender: TObject);
    procedure FileSaveAsAccept(Sender: TObject);
    procedure AnalyzeClick(Sender: TObject);
    procedure SynthesizeClick(Sender: TObject);
    procedure ParametersChanged(Sender: TObject);
    procedure mniExitClick(Sender: TObject);
  private
    { Private declarations }
    Signal, SynthSig: TVector;
    Decomp: TDecomposition;
    SignalChart, ApproxChart: TChart;
    DetailsCharts: array of TChart;
    SignalSeries, ApproxSeries, SynthSeries: TChartSeries;
    DetailsSeries: array of TChartSeries;
    function AddChart(const AAlign: TAlign; const ACaption: string): TChart;
    function AddSeries(const AChart: TChart; const AData: TVector; Level: Integer): TChartSeries;
    procedure ClearAll;
    procedure ClearDecomp;
  public
    { Public declarations }
  end;

var
  TestForm: TTestForm;

implementation

{$R *.DFM}

function TTestForm.AddChart(const AAlign: TAlign; const ACaption: string): TChart;
begin
  Result := TChart.Create(Self);
  with Result do
  begin
    Height := 200;
    Align := AAlign;
    View3D := False;
    Legend.Visible := False;
    Title.Text.Text := ACaption;
    Parent := ChartBox;
  end;
end;

function TTestForm.AddSeries(const AChart: TChart; const AData: TVector; Level: Integer): TChartSeries;
var
  NormCoef: Double;
  I: Integer;
begin
  if Level > 0 then NormCoef := Exp(-Level*Ln(Sqrt(2))) else NormCoef := 1;
  Result := TFastLineSeries.Create(Self);
  AChart.AddSeries(Result);
  for I := 0 to High(AData) do Result.AddY(AData[I]*NormCoef,'',clTeeColor);
end;

procedure TTestForm.ClearAll;
begin
  ClearDecomp;
  Analyze.Enabled := False;
  SignalSeries.Free;
  SignalChart.Free;
  SignalSeries := nil;
  SignalChart := nil;
  Signal := nil;
end;

procedure TTestForm.ClearDecomp;
var
  I: Integer;
begin
  Synthesize.Enabled := False;
  with Decomp do
  begin
    for I := 0 to High(Details) do
    begin
      DetailsSeries[I].Free;
      DetailsCharts[I].Free;
      Details[I] := nil;
    end;
    DetailsSeries := nil;
    DetailsCharts := nil;
    Details := nil;
    ApproxSeries.Free;
    ApproxChart.Free;
    ApproxSeries := nil;
    ApproxChart := nil;
    Approx := nil;
  end;
  SynthSeries.Free;
  SynthSeries := nil;
  SynthSig := nil;
end;

procedure TTestForm.FormDestroy(Sender: TObject);
begin
  ClearAll;
end;

procedure TTestForm.FileOpenAccept(Sender: TObject);
var
  FileName: string;
  F: TextFile;
begin
  ClearAll;
  if not OpenDialog.Execute then Exit;
  FileName := OpenDialog.FileName;
  AssignFile(F,FileName);
  Reset(F);
  try
    while not Eof(F) do
    begin
      StatusBar.SimpleText := Format(
        'Loading file %s: %d of %d bytes',
        [FileName,FilePos(F),FileSize(F)]);
      SetLength(Signal,Succ(Length(Signal)));
      Read(F,Signal[High(Signal)]);
    end;
    StatusBar.SimpleText := 'Ready';
    Analyze.Enabled := True;
  finally
    CloseFile(F);
  end;
  SignalChart := AddChart(alClient,FileName);
  SignalSeries := AddSeries(SignalChart,Signal,0);
end;

procedure TTestForm.FileSaveAsAccept(Sender: TObject);
var
  F: TextFile;
  I: Integer;
begin
  if not SaveDialog.Execute then Exit;
  AssignFile(F,SaveDialog.FileName);
  Rewrite(F);
  try
    for I := 0 to High(SynthSig) do WriteLn(F,SynthSig[I]);
  finally
    CloseFile(F);
  end;
end;

procedure TTestForm.AnalyzeClick(Sender: TObject);
var
  I: Integer;
begin
  ClearDecomp;
  Analyze.Enabled := False;
  try
    //  
    Decomp := Decompose(Signal,WaveletOrder.Value,DecompLevel.Value);
    //  
    with Decomp do
    begin
      SignalChart.Align := alTop;
      SignalChart.Height := 200;
      ApproxChart := AddChart(alTop,'Approximation on level '+IntToStr(Length(Details)));
      ApproxSeries := AddSeries(ApproxChart,Approx,Length(Details));
      SignalChart.Top := 0;
      SetLength(DetailsCharts,Length(Details));
      SetLength(DetailsSeries,Length(Details));
      for I := 0 to High(Details) do
      begin
        DetailsCharts[I] := AddChart(alBottom,'Details on level '+IntToStr(Succ(I)));
        DetailsSeries[I] := AddSeries(DetailsCharts[I],Details[I],Succ(I));
      end;
    end;
    //  
    Synthesize.Enabled := True;
  finally
    Analyze.Enabled := True;
  end;
end;

procedure TTestForm.SynthesizeClick(Sender: TObject);
begin
  Synthesize.Enabled := False;
  Analyze.Enabled := False;
  try
    SynthSig := Denoise(Decomp);
    SynthSeries := AddSeries(SignalChart,SynthSig,0);
  finally
    Analyze.Enabled := True;
  end;
end;

procedure TTestForm.ParametersChanged(Sender: TObject);
begin
  ClearDecomp;
end;

procedure TTestForm.mniExitClick(Sender: TObject);
begin
  Close;
end;

end.
